import { FetchOptions, FetchResponse, $Fetch } from 'ofetch';

type RequestInfo = string | FetchConfig;
type MakeRequired<Type, Key extends keyof Type> = Omit<Type, Key> & Required<Pick<Type, Key>>;
type MakeOptional<Type, Key extends keyof Type> = Omit<Type, Key> & Partial<Pick<Type, Key>>;
interface FetchInterceptorOptions {
    onFulfilled?: (value: any) => any | Promise<any>;
    onRejected?: (error: any) => any;
    synchronous?: boolean;
    runWhen?: (config: FetchConfig) => boolean | null;
}
interface FetchInterceptorManager<V> {
    use<T = V>(onFulfilled?: (value: V) => T | Promise<T>, onRejected?: (error: any) => any, options?: FetchInterceptorOptions): number;
    eject(id: number): void;
    forEach(fn: (handler: FetchInterceptorOptions) => void): void;
}
interface FetchConfig extends FetchOptions {
    url?: any;
    timeout?: number;
    raw?: boolean;
    native?: boolean;
    xsrfCookieName?: string;
    xsrfHeaderName?: string;
    onUploadProgress?: ((progress?: {
        loaded?: number | null;
        total?: number | null;
    }) => void);
    /**
     * Only works when `native: true`
     */
    onDownloadProgress?: ((progress?: {
        lengthComputable?: boolean;
        loaded?: number | null;
        total?: number | null;
    }) => void);
}

declare class FetchInstance {
    #private;
    interceptors: {
        request: FetchInterceptorManager<FetchConfig>;
        response: FetchInterceptorManager<Promise<any>>;
    };
    constructor(config?: FetchConfig);
    native(request: RequestInfo, config?: FetchConfig): Promise<Response>;
    raw(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    request(request: RequestInfo, config?: FetchConfig): Promise<any>;
    getFetch(): $Fetch;
    getDefaults(): FetchConfig;
    getBaseURL(): string | undefined;
    setBaseURL(baseURL: string): void;
    setHeader(name: string, value: string | null): void;
    setToken(token: string, type: string): void;
    onRequest(fn: (config: FetchConfig) => any): number;
    onResponse(fn: (response: any) => any): number;
    onRequestError(fn: (error: any) => any): number;
    onResponseError(fn: (error: any) => any): number;
    create(options: FetchConfig): FetchInstance;
}
declare interface FetchInstance {
    $get(request: RequestInfo, config?: FetchConfig): Promise<any>;
    $head(request: RequestInfo, config?: FetchConfig): Promise<any>;
    $delete(request: RequestInfo, config?: FetchConfig): Promise<any>;
    $post(request: RequestInfo, config?: FetchConfig): Promise<any>;
    $put(request: RequestInfo, config?: FetchConfig): Promise<any>;
    $patch(request: RequestInfo, config?: FetchConfig): Promise<any>;
    $options(request: RequestInfo, config?: FetchConfig): Promise<any>;
    get(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    head(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    delete(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    post(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    put(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    patch(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
    options(request: RequestInfo, config?: FetchConfig): Promise<FetchResponse<any>>;
}
declare function createInstance(config?: FetchConfig): FetchInstance;
declare const $fetch: typeof createInstance;

export { $fetch, type FetchConfig, FetchInstance, type FetchInterceptorManager, type FetchInterceptorOptions, type MakeOptional, type MakeRequired, type RequestInfo, createInstance };
